#ifndef baseModelSet_h
#define baseModelSet_h

#include <vector>
#include "trainingSet.h"

/** This class holds a set of models with the same or different algorithms. */

class baseModelSet {
public:
  /** Train on a specified set, causes creation if not created */
  virtual bool train(std::vector<trainingExample> trainingSet) = 0;
  /** reset to pre-training state */
  virtual bool initialize() = 0;
  /** run regression or classification for each model */
  virtual std::vector<double> process(std::vector<double> inputVector) = 0;
  
  virtual ~baseModelSet() {};
  
#ifndef EMSCRIPTEN //The javascript code will do its own JSON parsing
public:
  /** Get a JSON representation of the model in the form of a styled string */
  virtual std::string getJSON() = 0;
  /** Write a JSON model description to specified file path */
  virtual void writeJSON(std::string filepath) = 0;
  /** configure empty model with string. See getJSON() */
  virtual bool putJSON(std::string jsonMessage) = 0;
  /** read a JSON file at file path and build a modelSet from it */
  virtual bool readJSON(std::string filepath) = 0;
  
#endif
};

#endif /* baseModelSet_h */
