#ifndef xmmTool_h
#define xmmTool_h

// PROBLEM : GMM / HHMM are equivalent to modelSet, not Model
// => should we make modelSet virtual ?
//    or change the interface of baseModel ?
//    or fork / rewrite XMM ? ===> DEFINITELY NOT !
// maybe an "intermediate" class ... ?

#include "baseModelSet.h"
//#include "../../dependancies/xmm/src/xmm.h"
#include "xmm.h"

#include <iostream>
/*
  How do we want to instantiate the classes ?

  --> xmmToolBase *gmm = new Gmm(); // classification
  --> '' *gmr = new Gmr(); // regression
  --> '' *hhmm = new Hhmm(); // classification (/ following ?)
  --> '' *hhmr = new Hhmr(); // regression

  We should be able to write :

  class Gmm : public xmmTool<xmm::GMM>, public modelSet {
    Gmm() :
    model(false) {
      // instantiate things ?
    }
  };
*/

// not useful for the moment, but might be in the future

class xmmToolBase {
public:
  virtual ~xmmToolBase() {}
  virtual void initialize() = 0;
  virtual void train(std::vector<trainingExample> set) = 0;
  virtual std::vector<double> process(std::vector<double> observation) = 0;  
};

//============================//

template<typename Model>
class xmmTool : public xmmToolBase {

protected:
  xmmTool(bool bi = false) : bimodal(bi) {
//    xmm::Multimodality multimodality = bi
//                                     ? xmm::Multimodality::Bimodal
//                                     : xmm::Multimodality::Unimodal;
//    model = Model(xmm::MemoryMode::OwnMemory, multimodality);
    model = Model(bi);
    model.configuration.multithreading = xmm::MultithreadingMode::Sequential;
    model.configuration.multiClass_regression_estimator = xmm::MultiClassRegressionEstimator::Mixture;
    model.configuration.gaussians.set(1);
    model.configuration.relative_regularization.set(0.001);
    model.configuration.absolute_regularization.set(0.001);
    model.configuration.changed = true;
    //std::cout << "rel reg : " << model.configuration.relative_regularization.get() << std::endl;
    //std::cout << "abs reg : " << model.configuration.absolute_regularization.get() << std::endl;
  }

public :
  virtual ~xmmTool() {} // if we want to use polymorphism and destroy from base class ptr (...?)

  virtual void initialize() {
    // TODO :
  }

  virtual void train(std::vector<trainingExample> set) {
    // reinit internal training set
    this->set = xmm::TrainingSet(xmm::MemoryMode::OwnMemory,
      this->bimodal
      ? xmm::Multimodality::Bimodal
      : xmm::Multimodality::Unimodal);
    int dim_in = static_cast<int>(set[0].input.size());
    int dim_out = static_cast<int>(set[0].output.size());
    if (this->bimodal) {
      this->set.dimension.set(dim_in + dim_out);
      this->set.dimension_input.set(dim_in);
    } else {
      this->set.dimension.set(dim_in);
      this->set.dimension_input.set(0);
    }
    
    std::vector<std::string> cn;
    for (int i = 0; i < this->set.dimension.get(); ++i) {
      cn.push_back("a");
    }
    this->set.column_names.set(cn); // is it really necessary ?
    
    // do the remaining part in the specialized methods after calling this one
  }

  virtual std::vector<double> process(std::vector<double> observation) = 0;

protected:
  bool bimodal;
  Model model;
  xmm::TrainingSet set;
};

//=============================//

class xmmGmm : public xmmTool<xmm::GMM> {
public:
  xmmGmm() : xmmTool(false) {}
  ~xmmGmm() {}
  
  void train(std::vector<trainingExample> set) {
    xmmTool::train(set);
    
    xmm::Phrase p = xmm::Phrase(xmm::MemoryMode::OwnMemory,
                                xmm::Multimodality::Unimodal);

    for (int i = 0; i < set.size(); ++i) {
      p.dimension.set(static_cast<unsigned int>(set[i].input.size()));
      
      std::vector<float> obsIn(set[i].input.begin(), set[i].input.end());
      p.record(obsIn);
      
      std::string lab = std::to_string(set[i].output[0]);
      p.label.set(std::to_string(set[i].output[0]));
      
      this->set.addPhrase(static_cast<int>(this->set.size()), p);
      
      p.clear();
    }
    
    model.train(&this->set);
    model.reset();
  }

  std::vector<double> process(std::vector<double> observation) {
    std::vector<float> fIn(observation.begin(), observation.end());
    this->model.filter(fIn);

    std::vector<double> dOut = { std::stod(this->model.results.likeliest) };
    return dOut;
  }

private:
  xmm::Results<xmm::GMM> results;
};

//=============================//

class xmmGmr : public xmmTool<xmm::GMM> {
public:
  xmmGmr() : xmmTool(true) {}
  ~xmmGmr() {}
  
  void train(std::vector<trainingExample> set) {
    xmmTool::train(set);

    xmm::Phrase p = xmm::Phrase(xmm::MemoryMode::OwnMemory,
                                xmm::Multimodality::Bimodal);
    
    p.dimension_input.set(static_cast<unsigned int>(set[0].input.size()));
    p.dimension.set(p.dimension_input.get() + static_cast<unsigned int>(set[0].output.size()));
    p.label.set("a");

    for (int i = 0; i < set.size(); ++i) {
      std::vector<float> obsIn(set[i].input.begin(), set[i].input.end());
      std::vector<float> obsOut(set[i].output.begin(), set[i].output.end());
      p.record_input(obsIn);
      p.record_output(obsOut);
    }
    
    this->set.addPhrase(static_cast<int>(this->set.size()), p);

    model.train(&this->set);
    model.reset();
  }
  
  std::vector<double> process(std::vector<double> observation) {
    std::vector<float> fIn(observation.begin(), observation.end());
    this->model.filter(fIn);

    std::vector<float> *dRes = &this->model.results.output_values;
    std::vector<double> dOut(dRes->begin(), dRes->end());
    return dOut;
  }

private:
  xmm::Results<xmm::GMM> results;
};

//=============================//

class xmmHhmm : public xmmTool<xmm::HierarchicalHMM> {
public:
  xmmHhmm() : xmmTool(false) {}
  ~xmmHhmm() {}

  std::vector<double> process(std::vector<double> observation) {
    //    this->results = this->model.filter(observation);
    this->model.filter(*reinterpret_cast<std::vector<float> *>(&observation));
    return this->model.results.smoothed_normalized_likelihoods; // or smoothed log
  }

private:
  xmm::Results<xmm::HMM> results;
};

//=============================//

class xmmHhmr : public xmmTool<xmm::HierarchicalHMM> {
public:
  xmmHhmr() : xmmTool(true) {}
  ~xmmHhmr() {}

  std::vector<double> process(std::vector<double> observation) {
    this->model.filter(*reinterpret_cast<std::vector<float> *>(&observation));
    return *reinterpret_cast<std::vector<double> *>(&this->model.results.output_values);
  }

private:
  xmm::Results<xmm::HMM> results;
};


#endif /* xmmTool_h */
